@@ -3,6 +3,7 @@
 #include "types.h"
 #include "tables.h"
 #include "l3loop.h"
+#include "layer3.h"
 #include "huffman.h"
 #include "bitstream.h"
 #include "l3bitstream.h"
@@ -11,22 +12,23 @@
 #define e        2.71828182845
 #define CBLIMIT  21
 #define SFB_LMAX 22
-
-int *scalefac_band_long  = &shine_scale_fact_band_index[3].l[0];
+#define en_tot_krit 10
+#define en_dif_krit 100
+#define en_scfsi_band_krit 10
+#define xm_scfsi_band_krit 10
 
 static void calc_scfsi(shine_psy_xmin_t *l3_xmin, int ch, int gr, shine_global_config *config);
-static int part2_length(shine_scalefac_t *scalefac, int gr, int ch, shine_side_info_t *si);
-static int bin_search_StepSize(int desired_rate, int ix[samp_per_frame2], gr_info * cod_info, shine_global_config *config);
-static int count_bit(int ix[samp_per_frame2], unsigned int start, unsigned int end, unsigned int table );
-static int bigv_bitcount(int ix[samp_per_frame2], gr_info *gi);
-static int new_choose_table( int ix[samp_per_frame2], unsigned int begin, unsigned int end );
-static void bigv_tab_select( int ix[samp_per_frame2], gr_info *cod_info );
-static void subdivide(gr_info *cod_info);
-static int count1_bitcount( int ix[ samp_per_frame2 ], gr_info *cod_info );
-static void calc_runlen( int ix[samp_per_frame2], gr_info *cod_info );
+static int part2_length(int gr, int ch, shine_global_config *config);
+static int bin_search_StepSize(int desired_rate, int ix[GRANULE_SIZE], gr_info * cod_info, shine_global_config *config);
+static int count_bit(int ix[GRANULE_SIZE], unsigned int start, unsigned int end, unsigned int table );
+static int bigv_bitcount(int ix[GRANULE_SIZE], gr_info *gi);
+static int new_choose_table( int ix[GRANULE_SIZE], unsigned int begin, unsigned int end );
+static void bigv_tab_select( int ix[GRANULE_SIZE], gr_info *cod_info );
+static void subdivide(gr_info *cod_info, shine_global_config *config );
+static int count1_bitcount( int ix[ GRANULE_SIZE ], gr_info *cod_info );
+static void calc_runlen( int ix[GRANULE_SIZE], gr_info *cod_info );
 static void calc_xmin(shine_psy_ratio_t *ratio, gr_info *cod_info, shine_psy_xmin_t *l3_xmin, int gr, int ch );
-static int quantize(int ix[samp_per_frame2], int stepsize, shine_global_config *config);
-static int ix_max( int ix[samp_per_frame2], unsigned int begin, unsigned int end );
+static int quantize(int ix[GRANULE_SIZE], int stepsize, shine_global_config *config);
 
 /*
  * shine_inner_loop:
@@ -34,7 +36,7 @@ static int ix_max( int ix[samp_per_frame2], unsigned int begin, unsigned int end
  * The code selects the best quantizerStepSize for a particular set
  * of scalefacs.
  */
-int shine_inner_loop(int ix[samp_per_frame2],
+int shine_inner_loop(int ix[GRANULE_SIZE],
                int max_bits, gr_info *cod_info, int gr, int ch,
                shine_global_config *config )
 {
@@ -48,7 +50,7 @@ int shine_inner_loop(int ix[samp_per_frame2],
 
     calc_runlen(ix,cod_info);                        /* rzero,count1,big_values*/
     bits = c1bits = count1_bitcount(ix,cod_info);    /* count1_table selection*/
-    subdivide(cod_info);                             /* bigvalues sfb division */
+    subdivide(cod_info, config);                     /* bigvalues sfb division */
     bigv_tab_select(ix,cod_info);                    /* codebook selection*/
     bits += bvbits = bigv_bitcount( ix, cod_info );  /* bit count */
   }
@@ -66,22 +68,19 @@ int shine_inner_loop(int ix[samp_per_frame2],
 
 int shine_outer_loop( int max_bits,
                        shine_psy_xmin_t  *l3_xmin, /* the allowed distortion of the scalefactor */
-                       int ix[samp_per_frame2], /* vector of quantized values ix(0..575) */
+                       int ix[GRANULE_SIZE], /* vector of quantized values ix(0..575) */
                        int gr, int ch, shine_global_config *config)
 {
   int bits, huff_bits;
-  shine_scalefac_t *scalefac   = &config->scalefactor;
   shine_side_info_t *side_info = &config->side_info; 
   gr_info *cod_info = &side_info->gr[gr].ch[ch].tt;
 
   cod_info->quantizerStepSize = bin_search_StepSize(max_bits,ix,cod_info, config);
 
-  cod_info->part2_length = part2_length(scalefac,gr,ch,side_info);
+  cod_info->part2_length = part2_length(gr,ch,config);
   huff_bits = max_bits - cod_info->part2_length;
 
   bits = shine_inner_loop(ix, huff_bits, cod_info, gr, ch, config );
-
-  cod_info->part2_length   = part2_length(scalefac,gr,ch,side_info);
   cod_info->part2_3_length = cod_info->part2_length + bits;
 
   return cod_info->part2_3_length;
@@ -99,22 +98,20 @@ void shine_iteration_loop(shine_global_config *config)
   int ch, gr, i;
   int *ix;
 
-  scalefac_band_long  = &shine_scale_fact_band_index[config->mpeg.samplerate_index + 3].l[0];
-
   for(ch=config->wave.channels; ch--; )
   {
-    for(gr=0; gr<2; gr++)
+    for(gr=0; gr<config->mpeg.granules_per_frame; gr++)
     {
       /* setup pointers */
-      ix = config->l3_enc[gr][ch];
-      config->l3loop.xr = config->mdct_freq[gr][ch];
+      ix = config->l3_enc[ch][gr];
+      config->l3loop.xr = config->mdct_freq[ch][gr];
 
       /* Precalculate the square, abs,  and maximum,
        * for use later on.
        */
-      for (i=samp_per_frame2, config->l3loop.xrmax=0; i--;)
+      for (i=GRANULE_SIZE, config->l3loop.xrmax=0; i--;)
       {
-        config->l3loop.xrsq[i] = mulsr(config->l3loop.xr[i],config->l3loop.xr[i]);
+        config->l3loop.xrsq[i]  = mulsr(config->l3loop.xr[i],config->l3loop.xr[i]);
         config->l3loop.xrabs[i] = labs(config->l3loop.xr[i]);
         if(config->l3loop.xrabs[i]>config->l3loop.xrmax)
           config->l3loop.xrmax=config->l3loop.xrabs[i];
@@ -125,14 +122,15 @@ void shine_iteration_loop(shine_global_config *config)
 
       calc_xmin(&config->ratio, cod_info, &l3_xmin, gr, ch );
 
-      calc_scfsi(&l3_xmin,ch,gr,config);
+      if ( config->mpeg.version == MPEG_I )
+        calc_scfsi(&l3_xmin,ch,gr,config);
 
       /* calculation of number of available bit( per granule ) */
-      max_bits = shine_ResvMaxBits(&config->pe[gr][ch],config);
+      max_bits = shine_max_reservoir_bits(&config->pe[ch][gr],config);
 
       /* reset of iteration variables */
-      memset(config->scalefactor.l[gr][ch],0,22);
-      memset(config->scalefactor.s[gr][ch],0,14);
+      memset(config->scalefactor.l[gr][ch],0,sizeof(config->scalefactor.l[gr][ch]));
+      memset(config->scalefactor.s[gr][ch],0,sizeof(config->scalefactor.s[gr][ch]));
 
       for ( i=4; i--; )
         cod_info->slen[i] = 0;
@@ -175,35 +173,31 @@ void calc_scfsi( shine_psy_xmin_t *l3_xmin, int ch, int gr,
 {
   shine_side_info_t *l3_side = &config->side_info;
   /* This is the scfsi_band table from 2.4.2.7 of the IS */
-  static int scfsi_band_long[5] = { 0, 6, 11, 16, 21 };
-
-#define en_tot_krit 10
-#define en_dif_krit 100
-#define en_scfsi_band_krit 10
-#define xm_scfsi_band_krit 10
+  static const int scfsi_band_long[5] = { 0, 6, 11, 16, 21 };
 
   int scfsi_band;
   unsigned scfsi_set;
 
   int sfb, start, end, i;
   int condition = 0;
-  long temp;
-  /* gr_info *cod_info = &l3_side->gr[gr].ch[ch].tt; */ /* Unused */
+  int temp;
 
-/*
- note. it goes quite a bit faster if you uncomment the next bit and exit
-       early from scfsi, but you then loose the advantage of common scale factors.
+  const int *scalefac_band_long = &shine_scale_fact_band_index[config->mpeg.samplerate_index][0];
 
-      for(scfsi_band=0;scfsi_band<4;scfsi_band++)
-         l3_side->scfsi[ch][scfsi_band] = 0;
-      return;
-*/
+  /* note. it goes quite a bit faster if you uncomment the next bit and exit
+     early from scfsi, but you then loose the advantage of common scale factors.
+
+  for(scfsi_band=0;scfsi_band<4;scfsi_band++)
+    l3_side->scfsi[ch][scfsi_band] = 0;
+  return;
+
+  */
 
   config->l3loop.xrmaxl[gr] = config->l3loop.xrmax;
   scfsi_set = 0;
 
   /* the total energy of the granule */
-  for ( temp = 0, i =samp_per_frame2; i--;  )
+  for ( temp = 0, i =GRANULE_SIZE; i--;  )
     temp += config->l3loop.xrsq[i]>>10; /* a bit of scaling to avoid overflow, (not very good) */
   if ( temp )
     config->l3loop.en_tot[gr] = log((double)temp * 4.768371584e-7) / LN2; /* 1024 / 0x7fffffff */
@@ -277,25 +271,18 @@ void calc_scfsi( shine_psy_xmin_t *l3_xmin, int ch, int gr,
       for(scfsi_band=0;scfsi_band<4;scfsi_band++)
          l3_side->scfsi[ch][scfsi_band] = 0;
   } /* if gr == 1 */
-
 }
 
-/* these used in next two functions */
-static int slen1_tab[16] = { 0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4 };
-static int slen2_tab[16] = { 0, 1, 2, 3, 0, 1, 2, 3, 1, 2, 3, 1, 2, 3, 2, 3 };
-
 /*
  * part2_length:
  * -------------
  * calculates the number of bits needed to encode the scalefacs in the
  * main data block.
  */
-int part2_length(shine_scalefac_t *scalefac,
-                 int gr, int ch,
-                 shine_side_info_t *si)
+int part2_length(int gr, int ch, shine_global_config *config)
 {
   int slen1, slen2, bits;
-  gr_info *gi = &si->gr[gr].ch[ch].tt;
+  gr_info *gi = &config->side_info.gr[gr].ch[ch].tt;
 
   bits = 0;
 
@@ -303,16 +290,16 @@ int part2_length(shine_scalefac_t *scalefac,
     slen1 = slen1_tab[ gi->scalefac_compress ];
     slen2 = slen2_tab[ gi->scalefac_compress ];
 
-    if ( !gr || !(si->scfsi[ch][0]) )
+    if ( !gr || !(config->side_info.scfsi[ch][0]) )
       bits += (6 * slen1);
 
-    if ( !gr || !(si->scfsi[ch][1]) )
+    if ( !gr || !(config->side_info.scfsi[ch][1]) )
       bits += (5 * slen1);
 
-    if ( !gr || !(si->scfsi[ch][2]) )
+    if ( !gr || !(config->side_info.scfsi[ch][2]) )
       bits += (5 * slen2);
 
-    if ( !gr || !(si->scfsi[ch][3]) )
+    if ( !gr || !(config->side_info.scfsi[ch][3]) )
       bits += (5 * slen2);
   }
   return bits;
@@ -358,8 +345,6 @@ void shine_loop_initialise(shine_global_config *config)
 {
   int i;
 
-  config->side_info.main_data_begin = 0;
-
   /* quantize: stepsize conversion, fourth root of 2 table.
    * The table is inverted (negative power) from the equation given
    * in the spec because it is quicker to do x*y than x/y.
@@ -375,14 +360,14 @@ void shine_loop_initialise(shine_global_config *config)
        * In quantize, the long multiply does not shift it's result left one
        * bit to compensate.
        */
-      config->l3loop.steptabi[i] = (long)((config->l3loop.steptab[i]*2) + 0.5);
+      config->l3loop.steptabi[i] = (int32_t)((config->l3loop.steptab[i]*2) + 0.5);
   }
 
   /* quantize: vector conversion, three quarter power table.
    * The 0.5 is for rounding, the .0946 comes from the spec.
    */
   for(i=10000; i--;)
-    config->l3loop.int2idx[i] = (long)(sqrt(sqrt((double)i)*(double)i) - 0.0946 + 0.5);
+    config->l3loop.int2idx[i] = (int)(sqrt(sqrt((double)i)*(double)i) - 0.0946 + 0.5);
 }
 
 /*
@@ -391,9 +376,10 @@ void shine_loop_initialise(shine_global_config *config)
  * Function: Quantization of the vector xr ( -> ix).
  * Returns maximum value of ix.
  */
-int quantize(int ix[samp_per_frame2], int stepsize, shine_global_config *config )
+int quantize(int ix[GRANULE_SIZE], int stepsize, shine_global_config *config )
 {
-  int i, max, ln, scalei;
+  int i, max, ln;
+  int32_t scalei;
   double scale, dbl;
 
   scalei = config->l3loop.steptabi[stepsize+127]; /* 2**(-stepsize/4) */
@@ -403,7 +389,7 @@ int quantize(int ix[samp_per_frame2], int stepsize, shine_global_config *config
   if((mulr(config->l3loop.xrmax,scalei)) > 165140) /* 8192**(4/3) */
     max = 16384; /* no point in continuing, stepsize not big enough */
   else
-    for(i=0, max=0;i<samp_per_frame2;i++)
+    for(i=0, max=0;i<GRANULE_SIZE;i++)
     {
       /* This calculation is very sensitive. The multiply must round it's
        * result or bad things happen to the quality.
@@ -427,14 +413,14 @@ int quantize(int ix[samp_per_frame2], int stepsize, shine_global_config *config
     }
 
   return max;
- }
+}
 
 /*
  * ix_max:
  * -------
  * Function: Calculate the maximum of ix from 0 to 575
  */
-int ix_max( int ix[samp_per_frame2], unsigned int begin, unsigned int end )
+static inline int ix_max( int ix[GRANULE_SIZE], unsigned int begin, unsigned int end )
 {
   register int i;
   register int max = 0;
@@ -451,12 +437,12 @@ int ix_max( int ix[samp_per_frame2], unsigned int begin, unsigned int end )
  * Function: Calculation of rzero, count1, big_values
  * (Partitions ix into big values, quadruples and zeros).
  */
-void calc_runlen( int ix[samp_per_frame2], gr_info *cod_info )
+void calc_runlen( int ix[GRANULE_SIZE], gr_info *cod_info )
 {
   int i;
   int rzero = 0;
 
-  for ( i = samp_per_frame2; i > 1; i -= 2 )
+  for ( i = GRANULE_SIZE; i > 1; i -= 2 )
     if ( !ix[i-1] && !ix[i-2] )
       rzero++;
     else
@@ -480,7 +466,7 @@ void calc_runlen( int ix[samp_per_frame2], gr_info *cod_info )
  * ----------------
  * Determines the number of bits to encode the quadruples.
  */
-int count1_bitcount(int ix[samp_per_frame2], gr_info *cod_info)
+int count1_bitcount(int ix[GRANULE_SIZE], gr_info *cod_info)
 {
   int p, i, k;
   int v, w, x, y, signbits;
@@ -526,9 +512,9 @@ int count1_bitcount(int ix[samp_per_frame2], gr_info *cod_info)
  * ----------
  * presumable subdivides the bigvalue region which will use separate Huffman tables.
  */
-void subdivide(gr_info *cod_info)
+void subdivide(gr_info *cod_info, shine_global_config *config)
 {
-  static struct
+  static const struct
   {
     unsigned region0_count;
     unsigned region1_count;
@@ -559,47 +545,40 @@ void subdivide(gr_info *cod_info)
     {6, 7}, /* 22 bands */
   };
 
-  int scfb_anz = 0;
-  int bigvalues_region;
-
-  if ( !cod_info->big_values)
+  if (!cod_info->big_values)
   { /* no big_values region */
     cod_info->region0_count = 0;
     cod_info->region1_count = 0;
   }
   else
   {
+    const int *scalefac_band_long = &shine_scale_fact_band_index[config->mpeg.samplerate_index][0];
+    int bigvalues_region, scfb_anz, thiscount;
+
     bigvalues_region = 2 * cod_info->big_values;
-    {
-      int thiscount, index;
-      /* Calculate scfb_anz */
-      while ( scalefac_band_long[scfb_anz] < bigvalues_region )
-        scfb_anz++;
 
-      cod_info->region0_count = subdv_table[scfb_anz].region0_count;
-      thiscount = cod_info->region0_count;
-      index = thiscount + 1;
-      while ( thiscount && (scalefac_band_long[index] > bigvalues_region) )
-      {
-        thiscount--;
-        index--;
-      }
-      cod_info->region0_count = thiscount;
+    /* Calculate scfb_anz */
+    scfb_anz = 0;
+    while ( scalefac_band_long[scfb_anz] < bigvalues_region )
+      scfb_anz++;
 
-      cod_info->region1_count = subdv_table[scfb_anz].region1_count;
-      index = cod_info->region0_count + cod_info->region1_count + 2;
-      thiscount = cod_info->region1_count;
-      while ( thiscount && (scalefac_band_long[index] > bigvalues_region) )
-      {
-        thiscount--;
-        index--;
-      }
-      cod_info->region1_count = thiscount;
-      cod_info->address1 = scalefac_band_long[cod_info->region0_count+1];
-      cod_info->address2 = scalefac_band_long[cod_info->region0_count
-                                            + cod_info->region1_count + 2 ];
-      cod_info->address3 = bigvalues_region;
+    for (thiscount = subdv_table[scfb_anz].region0_count; thiscount; thiscount--) {
+      if (scalefac_band_long[thiscount + 1] <= bigvalues_region)
+        break;
     }
+    cod_info->region0_count = thiscount;
+    cod_info->address1 = scalefac_band_long[thiscount + 1];
+
+    scalefac_band_long += cod_info->region0_count + 1;
+
+    for (thiscount = subdv_table[scfb_anz].region1_count; thiscount; thiscount--) {
+      if (scalefac_band_long[thiscount + 1] <= bigvalues_region)
+        break;
+    }
+    cod_info->region1_count = thiscount;
+    cod_info->address2 = scalefac_band_long[thiscount + 1];
+
+    cod_info->address3 = bigvalues_region;
   }
 }
 
@@ -608,7 +587,7 @@ void subdivide(gr_info *cod_info)
  * ----------------
  * Function: Select huffman code tables for bigvalues regions
  */
-void bigv_tab_select( int ix[samp_per_frame2], gr_info *cod_info )
+void bigv_tab_select( int ix[GRANULE_SIZE], gr_info *cod_info )
 {
   cod_info->table_select[0] = 0;
   cod_info->table_select[1] = 0;
@@ -635,7 +614,7 @@ void bigv_tab_select( int ix[samp_per_frame2], gr_info *cod_info )
  * of the Huffman tables as defined in the IS (Table B.7), and will not work
  * with any arbitrary tables.
  */
-int new_choose_table( int ix[samp_per_frame2], unsigned int begin, unsigned int end )
+int new_choose_table( int ix[GRANULE_SIZE], unsigned int begin, unsigned int end )
 {
   int i, max;
   int choice[2];
@@ -737,7 +716,7 @@ int new_choose_table( int ix[samp_per_frame2], unsigned int begin, unsigned int
  * --------------
  * Function: Count the number of bits necessary to code the bigvalues region.
  */
-int bigv_bitcount(int ix[samp_per_frame2], gr_info *gi)
+int bigv_bitcount(int ix[GRANULE_SIZE], gr_info *gi)
 {
   int bits = 0;
   unsigned int table;
@@ -756,7 +735,7 @@ int bigv_bitcount(int ix[samp_per_frame2], gr_info *gi)
  * ----------
  * Function: Count the number of bits necessary to code the subregion.
  */
-int count_bit(int ix[samp_per_frame2],
+int count_bit(int ix[GRANULE_SIZE],
               unsigned int start,
               unsigned int end,
               unsigned int table )
@@ -764,7 +743,7 @@ int count_bit(int ix[samp_per_frame2],
   unsigned            linbits, ylen;
   register int        i, sum;
   register int        x,y;
-  struct huffcodetab *h;
+  const struct huffcodetab *h;
 
   if(!table)
     return 0;
@@ -829,37 +808,36 @@ int count_bit(int ix[samp_per_frame2],
  * with a call to bin_search gain defined below, which
  * returns a good starting quantizerStepSize.
  */
-int bin_search_StepSize(int desired_rate, int ix[samp_per_frame2],
+int bin_search_StepSize(int desired_rate, int ix[GRANULE_SIZE],
                         gr_info * cod_info, shine_global_config *config)
 {
-  int top,bot,next,last,bit;
+  int bit, next, count;
 
-  top  = -120;
-  bot  = 0;
-  next = top;
+  next  = -120;
+  count = 120;
 
-  do
-  {
-    last = next;
-    next = (top+bot) >> 1;
+  do {
+    int half = count / 2;
 
-      if (quantize(ix,next,config) > 8192)
+    if (quantize(ix, next + half, config) > 8192)
       bit = 100000;  /* fail */
     else
     {
-      calc_runlen(ix,cod_info);            /* rzero,count1,big_values */
+      calc_runlen(ix, cod_info);           /* rzero,count1,big_values */
       bit = count1_bitcount(ix, cod_info); /* count1_table selection */
-      subdivide(cod_info);                 /* bigvalues sfb division */
-      bigv_tab_select(ix,cod_info);        /* codebook selection */
-      bit += bigv_bitcount(ix,cod_info);   /* bit count */
+      subdivide(cod_info, config);         /* bigvalues sfb division */
+      bigv_tab_select(ix, cod_info);       /* codebook selection */
+      bit += bigv_bitcount(ix, cod_info);  /* bit count */
     }
 
-    if (bit>desired_rate)
-      top = next;
+    if (bit < desired_rate)
+      count = half;
     else
-      bot = next;
-  }
-  while((bit!=desired_rate) && abs(last-next)>1);
+    {
+      next += half;
+      count -= half;
+    }
+  } while (count > 1);
+
   return next;
 }
-
